"""

"""
project = "bolt-ta"
version = "0.3"
release = "0.3.5"
description = "A task runner written in Python"
copyright = "2016 Abantos"
author = "Isaac Rodriguez"
author_email = "oss.abantos@outlook.com"
url = "https://github.com/abantos/bolt"
license = "MIT"
keywords = "automation task tool development"
# https://pypi.org/pypi?%3Aaction=list_classifiers
classifiers = [
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Topic :: Software Development :: Build Tools",
    "Topic :: Utilities",
]
